import java.awt.BorderLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.io.PrintWriter;
import java.net.ServerSocket;
import java.net.Socket;
import java.util.ArrayList;
import java.util.List;
import javax.swing.*;
import javax.swing.border.EmptyBorder;

public class ChatConnectToServer {
    JTextArea eingehend;
    JTextField ausgehend;
    BufferedReader reader;
    PrintWriter writer;
    Socket sock;
    JFrame frame; // Declare frame here

    private String userName;
    private String serverIP;
    private int port;

    private List<Socket> connections; // Store all connection sockets

    private boolean stressTestRunning = false; // Flag to track if stress test is running

    private ServerSocket serverSocket; // Server socket for accepting incoming connections

    public static void main(String[] args) {
        ChatConnectToServer client = new ChatConnectToServer();
        client.initializeConnectionDetails();
    }

    private void initializeConnectionDetails() {
        JFrame initFrame = new JFrame("Connection Setup");
        JPanel initPanel = new JPanel(new GridLayout(0, 2, 2, 2));

        JTextField nameField = new JTextField(20);
        JTextField ipField = new JTextField(20);
        JTextField portField = new JTextField(20);

        initPanel.add(new JLabel("Name:"));
        initPanel.add(nameField);
        initPanel.add(new JLabel("Server IP:"));
        initPanel.add(ipField);
        initPanel.add(new JLabel("Port:"));
        initPanel.add(portField);

        int result = JOptionPane.showConfirmDialog(null, initPanel,
                "Please Enter Connection Details", JOptionPane.OK_CANCEL_OPTION, JOptionPane.PLAIN_MESSAGE);

        if (result == JOptionPane.OK_OPTION) {
            userName = nameField.getText();
            serverIP = ipField.getText();
            try {
                port = Integer.parseInt(portField.getText());
            } catch (NumberFormatException e) {
                JOptionPane.showMessageDialog(null, "Invalid port number. Please enter a numeric value.");
                System.exit(1);
            }
            los();
        } else {
            System.exit(0);
        }
    }

    public void los() {
        frame = new JFrame("Chat Client - " + userName); // Initialize frame here
        JPanel mainPanel = new JPanel(new BorderLayout(10, 10)); // added spacing
        mainPanel.setBorder(new EmptyBorder(10, 10, 10, 10)); // added padding

        eingehend = new JTextArea(15, 30);
        eingehend.setLineWrap(true);
        eingehend.setWrapStyleWord(true);
        eingehend.setEditable(false);
        JScrollPane scroller = new JScrollPane(eingehend);
        scroller.setVerticalScrollBarPolicy(ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS);
        scroller.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);

        ausgehend = new JTextField(30);
        JButton sendButton = new JButton("Send");
        sendButton.addActionListener(new SendenButtonListener());
        ausgehend.addActionListener(new SendenButtonListener()); //For Enter Key 


        JPanel southPanel = new JPanel(new BorderLayout());
        southPanel.add(ausgehend, BorderLayout.CENTER);
        southPanel.add(sendButton, BorderLayout.EAST);


        mainPanel.add(scroller, BorderLayout.CENTER);
        mainPanel.add(southPanel, BorderLayout.SOUTH);

        netzwerkEinrichten();

        Thread readerThread = new Thread(new EingehenderReader());
        readerThread.start();

        frame.setContentPane(mainPanel);
        frame.setSize(500, 600); // adjusted window size
        frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        frame.setVisible(true);
    }

    private void netzwerkEinrichten() {
        try {
            serverSocket = new ServerSocket(port); // Initialize the server socket
            sock = new Socket(serverIP, port); // Connect to the server
            InputStreamReader streamReader = new InputStreamReader(sock.getInputStream(), "UTF-8");
            reader = new BufferedReader(streamReader);
            writer = new PrintWriter(new OutputStreamWriter(sock.getOutputStream(), "UTF-8"), true);
            System.out.println("Netzwerkverbindung steht");
        } catch (IOException ex) {
            JOptionPane.showMessageDialog(null, "Could not connect to server: " + ex.getMessage());
            System.exit(1);
        }
    }

    public class SendenButtonListener implements ActionListener {
        public void actionPerformed(ActionEvent ev) {
            try {
                String messageToSend = userName + ": " + ausgehend.getText();
                writer.println(messageToSend);
                writer.flush();
            } catch (Exception ex) {
                ex.printStackTrace();
            }
            ausgehend.setText("");
            ausgehend.requestFocus();
        }
    }

    public class EingehenderReader implements Runnable {
        public void run() {
            String nachricht;
            try {
                while ((nachricht = reader.readLine()) != null) {
                    System.out.println("gelesen: " + nachricht);
                    eingehend.append(nachricht + "\n");
                }
            } catch (Exception ex) {
                ex.printStackTrace();
            }
        }
    }
}
